import ReactTimeAgo from 'react-time-ago'
import TimeAgo from 'javascript-time-ago'
import ru from 'javascript-time-ago/locale/ru.json'
import { useCallback, useState, useEffect, useRef } from 'react'

TimeAgo.addLocale(ru)

const ContextMenu = ({ element, message, owner, onEdit, onReplyTo, onForwardTo }) => {
    const [points, setPoints] = useState({ x: 0, y: 0 })
    const [show, setShow] = useState("none")

    const handler = useCallback(e => {
        e.preventDefault()
        setPoints({ x: e.pageX, y: e.pageY })
        setShow("flex")
    }, [setShow, setPoints])

    const clickHandler = useCallback(() => setShow("none"), [setShow]);

    useEffect(() => {
        if (element) {
            element.addEventListener("contextmenu", handler);
            element.addEventListener("click", clickHandler);
        }
        return () => {
            if (element) {
                element.removeEventListener("contextmenu", handler);
                element.removeEventListener("click", clickHandler);
            }
        }
    });

    return (
        <div
            className="context-menu"
            style={{
                display: show,
                top: points.y,
                left: points.x
            }}
        >
            {!owner && <button onClick={() => onReplyTo(message)}>Ответить</button>}
            <button onClick={() => onForwardTo(message)}>Переслать</button>
            {owner && <button onClick={() => onEdit(true)}>Редактировать</button>}
        </div>
    )
}


export const Message = ({ id, nick, msg, date, media, modified, own = false, replies, onMSGEdit, onMSGReply, onMSGForward }) => {
    let [isEdit, setIsEdit] = useState(false)
    let [messageEdit, setMessageEdit] = useState("" || msg)
    let el = useRef(null)

    let msgEditHandler = () => {
        setIsEdit(false)
        onMSGEdit(id, messageEdit)
    }

    return (
        <li ref={el} className={own ? "msg-user" : "msg-someone"} >
            <ContextMenu message={id} element={el.current} owner={own} onEdit={setIsEdit} onForwardTo={onMSGForward} onReplyTo={onMSGReply} />

            <span className="msg-nick">{nick}</span>

            {isEdit ?
                <>
                    <input value={messageEdit} onChange={(e) => setMessageEdit(e.target.value)} />
                    <button onClick={() => msgEditHandler()} >Сохранить изменения</button>
                </> :
                <span className="msg-text">{msg}</span>}

            <ReactTimeAgo className="msg-date" date={+date} locale="ru" timeStyle="round" />
            {modified && <span className="mutated-msg">Сообщение изменено</span>}
            {replies && replies.map(repMSG => <Message nick={repMSG.login} msg={repMSG.text} date={repMSG.createdAt} />)}
            {media && media.length > 0 &&
                <ul className="msg-media">
                    {media.map(file => {
                        return (<li key={file.url}>
                            {file.type.includes("audio") &&
                                <audio className="msg-media-audio" preload="metadata" controls>
                                    <source src={"/" + file.url} type={file.type} />
                                    <a href={"/" + file.url} />
                                </audio>
                            }

                            {
                                file.type.includes("image") &&
                                <img className="msg-media-img" src={"/" + file.url} alt="image" />
                            }

                            {
                                file.type.includes("video") &&
                                <video className="msg-media-video" preload="metadata" controls>
                                    <source src={"/" + file.url} type={file.type} />
                                    <a href={"/" + file.url} />
                                </video>
                            }
                        </li>)
                    })}
                </ul>}
        </li>
    )
}
